
const Cart = require('../model/Cart');
const { v4: uuidv4 } = require('uuid');
const jwt = require('jsonwebtoken');

require("dotenv").config()

exports.addToCart = async (req, res) => {
  let { userId, anonymousUserId } = req.body;
  const { productId, quantity } = req.body;

  if (!userId && !anonymousUserId) {
    return res.status(400).json({ error: 'User ID or anonymous ID is required' });
  }

  try {
    if (!userId && anonymousUserId) {
      // If anonymousUserId is present, create a separate entry in the database
      const cart = new Cart({
        anonymousUserId,
        items: [{ product: productId, quantity }],
      });

      await cart.save();

      console.log(`Product with ID ${productId} added to cart for anonymous user with ID ${anonymousUserId}`);

      return res.json({ message: 'Item added to cart successfully', userId: anonymousUserId });
    }

    // If userId is provided, use it for the database query
    let cart = await Cart.findOne({ userId }).populate('items.product');

    if (!cart) {
      cart = new Cart({
        userId,
        items: [{ product: productId, quantity }],
      });
    } else {
      const existingItem = cart.items.find(item => item.product._id.toString() === productId);

      if (existingItem) {
        existingItem.quantity += quantity;
      } else {
        cart.items.push({ product: productId, quantity });
      }
    }

    await cart.save();

    console.log(`Product with ID ${productId} added to cart for user with ID ${userId}`);

    res.json({ message: 'Item added to cart successfully', userId });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: 'Internal Server Error' });
  }
};


exports.updateCart = async (req, res) => {
  let { userId } = req.body;
  const { updates } = req.body;

  // If userId is 'anonymousUserId', use a constant user ID
  if (userId === 'anonymousUserId') {
    userId = 'constantAnonymousUserId';
  }

  try {
    if (!userId) {
      return res.status(400).json({ error: 'User ID is required' });
    }

    let cart = await Cart.findOne({ userId });

    if (!cart) {
      return res.status(404).json({ error: 'Cart not found for the user' });
    }

    updates.forEach((update) => {
      const { product, quantity } = update;

      const existingItem = cart.items.find(
        (item) => item.product.toString() === product.toString()
      );

      if (!existingItem) {
        return res.status(404).json({ error: 'Product not found in the cart' });
      }

      if (quantity === 0) {
        // If the quantity is 0, remove the product from the cart
        cart.items = cart.items.filter(
          (item) => item.product.toString() !== product.toString()
        );
      } else {
        existingItem.quantity = quantity;
      }
    });

    await cart.save();

    res.json({ message: 'Cart updated successfully', userId });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: 'Internal Server Error' });
  }
};

exports.removeFromCart = async (req, res) => {
  let { userId } = req.body;
  const { product } = req.body;

  // If userId is 'anonymousUserId', use a constant user ID
  if (userId === 'anonymousUserId') {
    userId = 'constantAnonymousUserId';
  }

  try {
    if (!userId) {
      return res.status(400).json({ error: 'User ID is required' });
    }

    let cart = await Cart.findOne({ userId });

    if (!cart) {
      return res.status(404).json({ error: 'Cart not found for the user' });
    }

    // Remove the product from the cart
    cart.items = cart.items.filter(item => item.product.toString() !== product);

    await cart.save();

    res.json({ message: 'Product removed from cart successfully', userId });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: 'Internal Server Error' });
  }
};

// Main getCartDetails function that handles both cases
exports.getCartDetails = async (req, res) => {
  const { userId } = req.params;
  try {
    const cart = await Cart.findOne({ userId });
    if (!cart) {
      return res.status(404).json({ message: 'Cart not found for the user' });

    }

    return res.json({ items: cart.items });
  } catch (error) {
    console.error('Error fetching cart details:', error);
    res.status(500).json({ error: 'Internal Server Error' });
  }
};

// Function to get cart details by anonymous user ID
const getCartDetailsByAnonymousUserId = async (userId) => {
  try {
    const cart = await Cart.findByAnonymousUserId(userId);
    
    console.log('Cart found:', cart);

    if (!cart) {
      return { status: 404, message: "Cart not found for the user" };
    }

    return cart;
  } catch (error) {
    throw error;
  }
};



// New function for anonymous user cart details
exports.getCartDetailsForAnonymousUser = async (req, res) => {
  try {
    const anonymousUserId = req.params.anonymousUserId;

    console.log("Received anonymousUserId:", anonymousUserId);

    if (anonymousUserId) {
      const cartDetails = await getCartDetailsByAnonymousUserId(anonymousUserId);
      if (!cartDetails) {
        return res.status(404).json({ message: "Cart not found for the user" });
      }

      res.json({ items: cartDetails.items });
    } else {
      res.status(400).json({ error: "Invalid request" });
    }
  } catch (error) {
    console.error("Error fetching cart details for anonymous user:", error);
    res.status(500).json({ error: "Internal Server Error" });
  }
};

exports.clearCart = async (req, res) => {
  try {
    const token = req.headers.authorization.split(' ')[1];
    const anonymousUserId = req.headers['x-anonymous-user-id'];

    // Check if the request is from an authenticated user
    if (token) {
      
      const decoded = jwt.verify(token, process.env.SECRET_SAUCE) // Replace 'your-secret-key' with your actual secret key
      const userId = decoded.userId;

      // Clear the cart for the authenticated user
      await Cart.findOneAndDelete({ userId });
    } else if (anonymousUserId) {
      // Clear the cart for the anonymous user
      await Cart.findOneAndDelete({ anonymousUserId });
    } else {
      return res.status(400).json({ error: 'Invalid request. Missing user information.' });
    }

    return res.status(200).json({ message: 'Cart cleared successfully.' });
  } catch (error) {
    console.error('Error clearing cart:', error);
    return res.status(500).json({ error: 'Internal server error.' });
  }
};