const Product = require("../model/Product")
const fs = require('fs');
require("dotenv").config();
const path = require('path');


exports.editProduct = async (req, res) => {
  try {
    const productId = req.params.productId;
    const updatedProductData = req.body;

    // Validate or sanitize the input data if needed

    // Find the product by ID and update it
    const updatedProduct = await Product.findByIdAndUpdate(
      productId,
      updatedProductData,
      { new: true } // This option ensures you get the updated product in the response
    );

    if (!updatedProduct) {
      return res.status(404).json({ message: 'Product not found' });
    }

    // Send the updated product in the response
    res.status(200).json(updatedProduct);
  } catch (error) {
    console.error('Error editing product:', error);
    res.status(500).json({ message: 'Internal server error' });
  }
};

exports.productTag = async (req, res) => {
  try {
    const { tag } = req.query;

    // If no specific tag is provided, return all products
    const filter = tag ? { tags: tag } : {};

    // Fetch products from the database based on the filter
    const products = await Product.find(filter);

    res.json(products); // Ensure this line sends a JSON array, even if empty ([] for no products).
  } catch (error) {
    console.error('Error fetching tagged products:', error);
    res.status(500).json({ error: 'Internal Server Error' });
  }
}

exports.createProduct = async (req, res) => {
  try {
    const { name, description, price, isActive } = req.body;

    if (req.file) {
      const imagePath = req.file.path;

      const newProduct = new Product({
        name,
        description,
        price,
        isActive,
        image: `http://localhost:4000/uploads/${path.basename(imagePath)}`, // Store the image link
      });

      await newProduct.save();

      return res.status(201).json(newProduct);
    } else {
      return res.status(400).json({ error: 'Image is required' });
    }
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: 'Internal server error' });
  }
};

exports.deleteProduct = async (req, res) => {
  const productId = req.params.id;

  try {
    // Find the product by ID
    const product = await Product.findById(productId);

    if (!product) {
      return res.status(404).json({ message: 'Product not found' });
    }

    // Remove the associated image file from the "uploads" folder
    if (product.image) {
      const imagePath = path.join(__dirname, '../uploads', path.basename(product.image));
      fs.unlinkSync(imagePath);
    }

    // Use deleteOne method to delete the product
    await product.deleteOne();

    res.json({ message: 'Product deleted successfully' });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: 'Internal Server Error' });
  }
};



// Controller function for retrieving all products (accessible to both admin and normal user)
exports.getAllProducts = async (req, res) => {
	try {
		const products = await Product.find()

		res.status(200).json(products)
	} catch (error) {
		console.error(error) 
		res.status(500).json({ message: "Internal server error" })
	}
}

// Controller function for retrieving all active products (accessible to both admin and normal user)
exports.getActiveProducts = async (req, res) => {
	try {
		const activeProducts = await Product.find({ isActive: true })

		res.status(200).json(activeProducts)
	} catch (error) {
		console.error(error)
		res.status(500).json({ message: "Internal server error" })
	}
}

// Controller to retrieve a single product by id
exports.getProductById = async (req, res) => {
  const productId = req.params.id;

  try {
    const product = await Product.findById(productId);

    if (!product) {
      return res.status(404).json({ message: 'Product not found' });
    }

    res.status(200).json(product);
  } catch (error) {
    console.error('Error fetching product by ID:', error);
    res.status(500).json({ message: 'Internal server error' });
  }
};

// Controller function to update product information
exports.updateProduct = async (req, res) => {
  const productId = req.params.id;
  const { name, description, price, isActive } = req.body;

  try {
    // Check if the product exists
    const existingProduct = await Product.findById(productId);
    if (!existingProduct) {
      return res.status(404).json({ message: 'Product not found' });
    }

    // Update product information
    existingProduct.name = name;
    existingProduct.description = description;
    existingProduct.price = price;
    existingProduct.isActive = isActive;

    // Save the updated product
    const updatedProduct = await existingProduct.save();

    res.status(200).json(updatedProduct);
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: 'Internal Server Error' });
  }
};

exports.activateProduct = async (req, res) => {
  const { productId } = req.params;

  try {
    // Find the product by ID
    const product = await Product.findById(productId);

    if (!product) {
      return res.status(404).json({ message: 'Product not found' });
    }

    // Check if the product is already active
    if (product.isActive) {
      return res.status(400).json({ message: 'Product is already active' });
    }

    // Activate the product
    product.isActive = true;

    // Save the updated product
    await product.save();

    res.json({ message: 'Product activated successfully', product });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: 'Internal Server Error' });
  }
};

exports.archiveProduct = async (req, res) => {
  const { productId } = req.params;

  try {
    // Find the product by ID
    const product = await Product.findById(productId);

    if (!product) {
      return res.status(404).json({ message: 'Product not found' });
    }

    // Check if the product is already active
    if (!product.isActive) {
      return res.status(400).json({ message: 'Product is already in Archive' });
    }

    // Activate the product
    product.isActive = false;

    // Save the updated product
    await product.save();

    res.json({ message: 'Product successfuly archived', product });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: 'Internal Server Error' });
  }
};
