import React, { useState, useEffect } from "react";
import {
  Container,
  Row,
  Col,
} from "react-bootstrap";
import OrderSummary from "../components/OrderSummary";
import PersonalInfoCollapse from "../components/PersonalInfoCollapse";
import AddressCollapse from "../components/AddressCollapse";
import PaymentCollapse from "../components/PaymentCollapse";
import ShippingMethodCollapse from "../components/ShippingMethodCollapse";
import { jwtDecode } from "jwt-decode";
import { toast, ToastContainer } from "react-toastify";
import "react-toastify/dist/ReactToastify.css";

const CheckoutPage = () => {
  const [cartItems, setCartItems] = useState([]);
  const [totalPrice, setTotalPrice] = useState(0);
  const [shippingFee] = useState({
    value: 200,
    details: "Fixed shipping fee of ₱200",
  });
  const [promoCode, setPromoCode] = useState("");
  const [personalInfoOpen, setPersonalInfoOpen] = useState(true);
  const [addressOpen, setAddressOpen] = useState(false);
  const [shippingMethodOpen, setShippingMethodOpen] = useState(false);
  const [isCustomerLoggedIn, setIsCustomerLoggedIn] = useState(false);
  const [userDetails, setUserDetails] = useState(null);
  const [appliedPromoCode, setAppliedPromoCode] = useState(null);
  const [productDetails, setProductDetails] = useState({});
  const [discountAmount, setDiscountAmount] = useState(0);
  const [addAddressFormOpen, setAddAddressFormOpen] = useState(false);
  const [addresses, setAddresses] = useState([]);
  const [addressFormError] = useState(null);
  const [selectedAddress, setSelectedAddress] = useState(null);
  const [selectedShippingMethod, setSelectedShippingMethod] = useState(null);

  const handleDeleteAddress = async (addressId) => {
    try {
      const token = localStorage.getItem("token");

      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/user/delete-address/${addressId}`,
        {
          method: "DELETE",
          headers: {
            Authorization: `Bearer ${token}`,
          },
        }
      );

      if (response.ok) {
        console.log("Address deleted successfully:", addressId);
        toast.success("Successfully deleted address", {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
        // Fetch addresses again after deleting an address
        fetchAddresses();
      } else {
        console.error("Failed to delete address:", response.status);
        toast.error("Failed to delete address", {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
      }
    } catch (error) {
      console.error("Error deleting address:", error);
      toast.error("Error deleting address", {
        position: "bottom-right",
        autoClose: 3000,
        hideProgressBar: false,
        closeOnClick: true,
        pauseOnHover: true,
        draggable: true,
      });
    }
  };

  const handleContinueAddress = async (e) => {
    e.preventDefault(); // Prevent the default form submission behavior

    // Add validation logic here to check if the required fields are filled
    if (
      !selectedAddress.street ||
      !selectedAddress.zipCode ||
      !selectedAddress.city ||
      !selectedAddress.state ||
      !selectedAddress.country
    ) {
      console.error("Please fill in all required address fields");
      // Display an error toast
      toast.error("Please fill in all required address fields", {
        position: "bottom-right",
        autoClose: 3000,
        hideProgressBar: false,
        closeOnClick: true,
        pauseOnHover: true,
        draggable: true,
      });
      return;
    }

    try {
      const token = localStorage.getItem("token");

      // Decode the token to extract user information
      const decodedToken = jwtDecode(token);
      const userId = decodedToken.userId;

      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/user/create-address`,
        {
          method: "POST",
          headers: {
            "Content-Type": "application/json",
            Authorization: `Bearer ${token}`,
          },
          body: JSON.stringify({ ...selectedAddress, userId }), // Include userId in the request body
        }
      );

      if (response.ok) {
        const data = await response.json();
        console.log("Address created successfully:", data);
        // Optionally, you can update the UI or state to reflect the new address
        // Display a success toast
        toast.success("Address created successfully!", {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
        // Fetch addresses again after creating a new address
        fetchAddresses();

        // Close the forms after creating a new address
        setAddAddressFormOpen(false);
        setPersonalInfoOpen(false);
      } else {
        console.error("Failed to create address:", response.status);
        // Display an error toast
        toast.error("Failed to create address", {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
      }
    } catch (error) {
      console.error("Error creating address:", error);
      // Display an error toast
      toast.error("Error creating address", {
        position: "bottom-right",
        autoClose: 3000,
        hideProgressBar: false,
        closeOnClick: true,
        pauseOnHover: true,
        draggable: true,
      });
    }
  };

  const fetchAddresses = async () => {
    try {
      const token = localStorage.getItem("token");

      // Decode the JWT token to get user information
      const decoded = jwtDecode(token);
      const userId = decoded.userId;

      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/user/get-address`,
        {
          method: "POST",
          headers: {
            Authorization: `Bearer ${token}`,
            "Content-Type": "application/json",
          },
          body: JSON.stringify({ userId }),
        }
      );

      if (response.ok) {
        const data = await response.json();
        setAddresses(data);

        return data; // Return the fetched addresses
      } else {
        console.error("Failed to fetch addresses:", response.status);
        return []; // Return an empty array if fetching fails
      }
    } catch (error) {
      console.error("Error fetching addresses:", error);
      return []; // Return an empty array if an error occurs
    }
  };

  const fetchCartItems = async () => {
    try {
      const token = localStorage.getItem("token");
      const decoded = jwtDecode(token);

      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/cart/user/${decoded.userId}`,
        {
          method: "GET",
          headers: {
            Authorization: `Bearer ${token}`,
          },
        }
      );

      const data = await response.json();

      if (Array.isArray(data.items)) {
        setCartItems(data.items);
      }
    } catch (error) {
      console.error("Error fetching cart items:", error);
    }
  };

  const calculateTotalPrice = () => {
    const totalPriceWithoutShipping = cartItems.reduce((total, item) => {
      const product = productDetails[item.product];
      if (product) {
        return total + product.price * item.quantity;
      }
      return total;
    }, 0);

    // Calculate the discount based on the percentage
    const calculatedDiscount =
      (discountAmount / 100) * totalPriceWithoutShipping;

    // Calculate the total price after applying the discount
    const totalPriceAfterDiscount =
      totalPriceWithoutShipping - calculatedDiscount;

    // Use shippingFee.value in the calculation
    const updatedTotalPrice = totalPriceAfterDiscount + shippingFee.value;

    // Update total price state
    setTotalPrice(updatedTotalPrice);
  };

  const fetchProductDetails = async () => {
    try {
      const token = localStorage.getItem("token");
      const productIds = cartItems.map((item) => item.product);

      const promises = productIds.map(async (productId) => {
        const response = await fetch(
          `${process.env.REACT_APP_API_URL}/product/${productId}`,
          {
            method: "GET",
            headers: {
              Authorization: `Bearer ${token}`,
            },
          }
        );

        if (!response.ok) {
          console.error(
            `Failed to fetch product details for ID ${productId}: ${response.status}`
          );
          return null;
        }

        const productData = await response.json();
        return { [productId]: productData };
      });

      const productDetailsArray = await Promise.all(promises);
      const productDetailsObject = productDetailsArray.reduce(
        (acc, product) => ({ ...acc, ...product }),
        {}
      );
      setProductDetails(productDetailsObject);
    } catch (error) {
      console.error("Error fetching product details:", error);
    }
  };

  const fetchUserProfile = async () => {
    try {
      if (isCustomerLoggedIn) {
        const token = localStorage.getItem("token");

        const response = await fetch(
          `${process.env.REACT_APP_API_URL}/user/details`,
          {
            method: "GET",
            headers: {
              Authorization: `Bearer ${token}`,
            },
          }
        );

        if (response.ok) {
          const data = await response.json();
          setUserDetails(data);
        } else {
          console.error("Failed to fetch user profile:", response.status);
        }
      }
    } catch (error) {
      console.error("Error fetching user profile:", error);
    }
  };

  const checkCustomerLoginStatus = () => {
    const token = localStorage.getItem("token");

    if (token) {
      try {
        const decoded = jwtDecode(token);

        if (decoded && decoded.userId) {
          setIsCustomerLoggedIn(true);
          fetchCartItems(); // Fetch cart items when the user logs in
          return;
        }
      } catch (error) {
        console.error("Error decoding token:", error);
      }
    }

    setIsCustomerLoggedIn(false);
  };

  const handleApplyPromoCode = async () => {
    try {
      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/user/apply-promo-code`,
        {
          method: "POST",
          headers: {
            "Content-Type": "application/json",
            Authorization: `Bearer ${localStorage.getItem("token")}`,
          },
          body: JSON.stringify({
            promoCode,
            discountAmount,
          }),
        }
      );

      if (response.ok) {
        const data = await response.json();

        // Set discount amount obtained from the response
        setDiscountAmount(data.discountAmount || 0);

        // Set the applied promo code
        setAppliedPromoCode(promoCode);

        // Display success notification at the bottom-right
        toast.success("Promo code applied successfully!", {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });

        checkCustomerLoginStatus();
        // Wait for state to update before calculating the total price
        setTimeout(() => {
          calculateTotalPrice();
        }, 0);
      } else {
        console.error("Failed to apply promo code:", response.status);

        // Display error notification at the bottom-right
        toast.error("Failed to apply promo code", {
          position: "bottom-right",
          autoClose: 3000,
          hideProgressBar: false,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
        // Handle error or provide feedback to the user
      }
    } catch (error) {
      console.error("Error applying promo code:", error);

      // Display error notification at the bottom-right
      toast.error("Error applying promo code", {
        position: "bottom-right",
        autoClose: 3000,
        hideProgressBar: false,
        closeOnClick: true,
        pauseOnHover: true,
        draggable: true,
      });
    }
  };

  useEffect(() => {
    checkCustomerLoginStatus();
  }, []);

  useEffect(() => {
    fetchUserProfile();
  }, [isCustomerLoggedIn]);

  useEffect(() => {
    fetchCartItems();
  }, []);

  useEffect(() => {
    fetchProductDetails();
  }, [cartItems]);

  useEffect(() => {
    calculateTotalPrice();
  }, [cartItems, productDetails, shippingFee]);

  useEffect(() => {
    fetchAddresses();
  }, [isCustomerLoggedIn]);

  
  

  return (
    <Container className="my-5 text-start">
      <h2>Checkout</h2>
      <Row>
        {/* Left side - Collapsible Forms */}
        <Col xs={12} md={8}>
          {/* Collapsible Table - Personal Information */}
          <PersonalInfoCollapse
            personalInfoOpen={personalInfoOpen}
            setPersonalInfoOpen={setPersonalInfoOpen}
            isCustomerLoggedIn={isCustomerLoggedIn}
            userDetails={userDetails}
          />
          {/* Collapsible Table - Address */}
          <AddressCollapse
            addressOpen={addressOpen}
            setAddressOpen={setAddressOpen}
            isCustomerLoggedIn={isCustomerLoggedIn}
            selectedAddress={selectedAddress}
            setSelectedAddress={setSelectedAddress}
            handleContinueAddress={handleContinueAddress}
            addAddressFormOpen={addAddressFormOpen}
            setAddAddressFormOpen={setAddAddressFormOpen}
            addressFormError={addressFormError}
            addresses={addresses}
            handleDeleteAddress={handleDeleteAddress}
          />

          {/* Collapsible Table - Shipping Method */}
          <ShippingMethodCollapse
            shippingMethodOpen={shippingMethodOpen}
            setShippingMethodOpen={setShippingMethodOpen}
            selectedShippingMethod={selectedShippingMethod}
            setSelectedShippingMethod={setSelectedShippingMethod}
          />

          {/* Collapsible Table - Payment */}
          <PaymentCollapse
            totalPrice={totalPrice}
            userDetails={userDetails}
            selectedShippingMethod={selectedShippingMethod}
            cartItems={cartItems}
            appliedPromoCode={appliedPromoCode}
            selectedAddress={selectedAddress}
          />
          
        </Col>

        {/* Right side - Order Summary */}
        <OrderSummary
          cartItems={cartItems}
          shippingFee={shippingFee}
          totalPrice={totalPrice}
          discountAmount={discountAmount}
          promoCode={promoCode}
          setPromoCode={setPromoCode}
          handleApplyPromoCode={handleApplyPromoCode}
          appliedPromoCode={appliedPromoCode}
        />
      </Row>
      <ToastContainer position="bottom-right" />
    </Container>
  );
};

export default CheckoutPage;
