import React, { useState, useEffect } from "react";
import { v4 as uuidv4 } from "uuid";
import { useParams } from "react-router-dom";
import { Container, Row, Col, Image, Button } from "react-bootstrap";
import { toast } from "react-toastify";
import "react-toastify/dist/ReactToastify.css";
import { BsDash, BsPlus } from "react-icons/bs";
import "./productPage.css";
import { jwtDecode } from "jwt-decode";

const ProductPage = () => {
  const { productId } = useParams();
  const [product, setProduct] = useState(null);
  const [quantity, setQuantity] = useState(1);
  const [error, setError] = useState(null);

  useEffect(() => {
    const fetchProduct = async () => {
      try {
        const response = await fetch(
          `${process.env.REACT_APP_API_URL}/product/${productId}`
        );

        if (!response.ok) {
          throw new Error("Failed to fetch product");
        }

        const data = await response.json();
        setProduct(data);
        setError(null);
      } catch (error) {
        console.error("Error fetching product by ID:", error);
        setError("Error fetching product. Please try again.");
      }
    };

    fetchProduct();
  }, [productId]);

  const handleAddToCart = async () => {
    try {
      const token = localStorage.getItem("token");
      let anonymousUserId = localStorage.getItem("anonymousUserId");
  
      const headers = {
        "Content-Type": "application/json",
      };
  
      let userId; // variable to hold either userId or anonymousUserId
  
      if (token) {
        headers.Authorization = `Bearer ${token}`;
        userId = jwtDecode(token).userId;
      } else if (!anonymousUserId) {
        // Generate a new anonymousUserId if it doesn't exist
        anonymousUserId = uuidv4();
        localStorage.setItem("anonymousUserId", anonymousUserId);
      }
  
      headers["X-Anonymous-User-ID"] = anonymousUserId;
      userId = null; // Set userId to null if not logged in
  
      const response = await fetch(
        `${process.env.REACT_APP_API_URL}/cart/add-to-cart`,
        {
          method: "POST",
          headers,
          body: JSON.stringify({
            userId,
            anonymousUserId,
            productId,
            quantity,
          }),
        }
      );
  
      const data = await response.json();
  
      if (response.ok) {
        toast.success(data.message, {
          position: "bottom-right",
          autoClose: 2000,
          hideProgressBar: true,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
      } else {
        toast.error(data.error || "Error adding to cart", {
          position: "bottom-right",
          autoClose: 2000,
          hideProgressBar: true,
          closeOnClick: true,
          pauseOnHover: true,
          draggable: true,
        });
      }
    } catch (error) {
      console.error("Error adding to cart:", error);
    }
  };
  
  const handleQuantityDecrease = () => {
    if (quantity > 1) {
      setQuantity(quantity - 1);
    }
  };

  const handleQuantityIncrease = () => {
    setQuantity(quantity + 1);
  };

  if (error) {
    return (
      <Container>
        <Row>
          <Col>
            <h2>Error</h2>
            <p>{error}</p>
          </Col>
        </Row>
      </Container>
    );
  }

  if (!product) {
    return (
      <Container>
        <Row>
          <Col>
            <p>Loading...</p>
          </Col>
        </Row>
      </Container>
    );
  }

  return (
    <Container className="mt-4">
      <Row>
        <Col md={6} className="d-flex justify-content-center">
          <Image
            src={product.image || "placeholder-image-url"}
            alt={product.name}
            fluid
            className="product-image"
          />
        </Col>
        <Col md={6} className="product-details">
          <h2 className="product-title">{product.name}</h2>
          <p className="product-id">ID: {product._id}</p>
          <div className="product-description">
            <p>
              <strong>Description:</strong>
            </p>
            <p>{product.description}</p>
            {/* Add more bullet points as needed */}
          </div>
          <p className="product-price">
            <p>
              <strong>Price:</strong>
            </p>
            PHP{" "}
            {new Intl.NumberFormat("en-PH", {
              style: "currency",
              currency: "PHP",
            }).format(product.price)}
          </p>
          <div className="quantity-selector">
            <Button variant="outline-primary" onClick={handleQuantityDecrease}>
              <BsDash className="rounded" />
            </Button>
            <span className="quantity-value">{quantity}</span>
            <Button variant="outline-primary" onClick={handleQuantityIncrease}>
              <BsPlus />
            </Button>
          </div>
          <div className="button-group mt-3">
            <Button variant="primary" onClick={handleAddToCart}>
              Add to Cart
            </Button>
          </div>
        </Col>
      </Row>
    </Container>
  );
};

export default ProductPage;
